#!/System/Library/Frameworks/Ruby.framework/Versions/1.8/usr/bin/ruby

require 'osx/cocoa'
include OSX

$libraryDirectoryPath = File.dirname(__FILE__)
$LOAD_PATH << $libraryDirectoryPath

require 'rubygems'
require 'json'
require 'set'

def usage
	print "Usage:\n"
	print "-icnsPath: path to an icns image whose contents should be verified [required]\n"
	print "\n"
	print "Example:\n"
	print "ruby icns_check.rb\n -icnsPath ~/Projects/Products/Release\\ -\\ Standard/Courier.app/Contents/Resources/Courier.icns\n-minimumOperatingSystem 10.6"
	print "\n"
	exit 1
end

if ARGV.count == 0
	usage
end

def start
	print "=========     ICNS VALIDATION STARTED     ==========\n"
	print "\n"
end

def finish
	print "\n"
	print "=========     ICNS VALIDATION SUCCESS     ==========\n"
	print "\n"
	exit 0
end

def fail(reason = nil, severity = :warning)
	print "\n"
	print "=========     ICNS VALIDATION FAILURE     ==========\n"
	unless reason.nil?
		print "error: #{reason}\n"
	end
	print "\n"
	
	if severity == :fatal
		exit 1
	end
	
	def finish
		fail nil, :fatal
	end
end


# Check arguments

icnsPath = NSUserDefaults.standardUserDefaults.stringForKey "icnsPath"
if icnsPath.nil?
	fail "icns path must be specified with -icnsPath", :fatal
end
unless icnsPath.pathExtension.isEqual_ "icns"
	fail "icns path must have the path extension 'icns'", :fatal
end
unless NSFileManager.defaultManager.fileExistsAtPath(icnsPath)
	fail "icns doesn't exist at specified path #{icnsPath}", :fatal
end
icnsLocation = NSURL.fileURLWithPath icnsPath

minimumOperatingSystem = NSUserDefaults.standardUserDefaults.stringForKey "minimumOperatingSystem"
if minimumOperatingSystem.nil?
	minimumOperatingSystem = "10.5"
end


# Utility Functions

def icnsRead(location)
	icnsReadInputFilePath = File.join $libraryDirectoryPath, "icns_read.m"
	icnsReadOutputFilePath = File.join $libraryDirectoryPath, "icns_read.o"
	
	compileResult = %x(xcrun clang #{icnsReadInputFilePath} -o #{icnsReadOutputFilePath} -framework Foundation)
	if compileResult.length != 0
		return { "error" => "couldn't compile #{icnsReadInputFilePath}" }
	end
	
	runResult = %x(exec '#{icnsReadOutputFilePath}' read '#{location.path}')
	
	JSON.parse(runResult)
end

def displayForTypes(types)
	types = types.map do |type|
		"'#{type}'"
	end
	types.join(' ')
end

#

start

print "Checking #{icnsPath}\n"


# Get types

fileContents = icnsRead(icnsLocation)

error = fileContents["error"]
unless error.nil?
	fail "error reading contents of file, #{error}", :fatal
end

includedTypes = fileContents["types"].map do |currentType|
	currentType["type"]
end

# Check that the ICNS contains all the types we expect

tenFive = %W(ICNS is32 s8mk il32 l8mk it32 t8mk ic09)
tenSix = tenFive
tenSeven = [ 'ICNS', 'TOC ', [ 'is32', 'icp4', ], [ 'il32', 'icp5', ], 'ic07', 'ic08', 'ic09', 'ic10', ]
tenEight = [ 'ICNS', 'TOC ', [ 'is32', 'icp4', ], 'ic11', [ 'il32', 'icp5', ], 'ic12', 'ic07', 'ic13', 'ic08', 'ic14', 'ic09', 'ic10', ]

requiredTypes = [
	{ "os" => "10.5", "types" => tenFive },
	{ "os" => "10.6", "types" => tenSix },
	{ "os" => "10.7", "types" => tenSeven },
	{ "os" => "10.8", "types" => tenEight }
]
requiredTypes.each do |pair|
	os = pair["os"]
	comparison = os.to_ns.compare_options_ minimumOperatingSystem, NSNumericSearch
	next if comparison == NSOrderedAscending
	
	print "Comparing against types required for #{os}\n"
	
	missingTypes = []
	
	requiredTypes = pair["types"]
	requiredTypes.each do |currentType|
		present = false
		if currentType.is_a? Array
			# At least one of the types in the required types must be present in includedTypes
			
			presentTypes = currentType.select do |currentRequiredType|
				includedTypes.include? currentRequiredType
			end
			
			present = presentTypes.size > 0
		elsif currentType.is_a? String
			present = includedTypes.include? currentType
		else
			raise "unknown type"
		end
		
		missingTypes << currentType unless present
	end
	
	next if missingTypes.size == 0
	
	fail "icns doesn't include all the required types for #{os}\n\trequired types: #{displayForTypes(requiredTypes)}\n\tincluded types: #{displayForTypes(includedTypes)}\n\n\tmissing types: #{displayForTypes(missingTypes)}"
end


#

finish
